@@ -25,6 +25,23 @@
 extern "C" {
 #endif
 
+/*
+ * Some of the basic types pointer to structures:
+ */
+/* xmlIO.h */
+typedef struct _xmlParserInputBuffer xmlParserInputBuffer;
+typedef xmlParserInputBuffer *xmlParserInputBufferPtr;
+
+typedef struct _xmlOutputBuffer xmlOutputBuffer;
+typedef xmlOutputBuffer *xmlOutputBufferPtr;
+
+/* parser.h */
+typedef struct _xmlParserInput xmlParserInput;
+typedef xmlParserInput *xmlParserInputPtr;
+
+typedef struct _xmlParserCtxt xmlParserCtxt;
+typedef xmlParserCtxt *xmlParserCtxtPtr;
+
 #define BASE_BUFFER_SIZE 4000
 
 /**
@@ -796,16 +813,19 @@ void		xmlNodeDump		(xmlBufferPtr buf,
 					 int level,
 					 int format);
 
-/* These are exported from xmlIO.h
- 
-int		xmlSaveFileTo		(xmlOutputBuffer *buf,
+int		xmlSaveFileTo		(xmlOutputBufferPtr buf,
 					 xmlDocPtr cur,
 					 const char *encoding);
-int             xmlSaveFormatFileTo     (xmlOutputBuffer *buf,
-                                         xmlDocPtr cur,
-                                         const char *encoding,
-                                         int format);
- */ 					 
+int             xmlSaveFormatFileTo     (xmlOutputBufferPtr buf,
+					 xmlDocPtr cur,
+				         const char *encoding,
+				         int format);
+void		xmlNodeDumpOutput	(xmlOutputBufferPtr buf,
+					 xmlDocPtr doc,
+					 xmlNodePtr cur,
+					 int level,
+					 int format,
+					 const char *encoding);
 
 int		xmlSaveFormatFileEnc    ( const char * filename,
 					  xmlDocPtr cur,
